
# tkListboxAutoScan --
# This procedure is invoked when the mouse leaves an entry window
# with button 1 down.  It scrolls the window up, down, left, or
# right, depending on where the mouse left the window, and reschedules
# itself as an "after" command so that the window continues to scroll until
# the mouse moves back into the window or the mouse button is released.
#
# Arguments:
# w -		The entry window.

array set bltHierBox {
    afterId ""
    space   off
    action  ""
    x       0
    y       0
    toggle  yes
}

proc blt::HierBoxAutoScan { w } {
    global bltHierBox
    if { ![winfo exists $w] } {
	return
    }
    set x $bltHierBox(x)
    set y $bltHierBox(y)
    if { $y >= [winfo height $w] } {
	$w yview scroll 1 units
    } elseif { $y < 0 } {
	$w yview scroll -1 units
    } 
    blt::HierBoxMotion $w $x $y
    set bltHierBox(afterId) [after 10 blt::HierBoxAutoScan $w]
}

proc blt::HierBoxSelect { widget x y } {
    set node [$widget nearest $x $y where]
    switch $where {
	"gadget" { 
	    $widget toggle $node 
	}
	"select" {
	    $widget selection clear 0 end
	    $widget selection set $node
	    $widget focus $node
	    $widget selection anchor $node 
	}
    }
}

proc blt::HierBoxStartSelect { widget node } {
    $widget selection anchor $node
}

proc blt::HierBoxMotion { widget x y } {
    set node [$widget nearest $x $y]
    global bltHierBox
    if { $bltHierBox(action) != "" } {
	$widget selection dragto $node $bltHierBox(action)
    }
}

proc blt::HierBoxEndSelect { widget x y } {
    set node [$widget nearest $x $y where]
    if { $node == "" } {
	$widget selection cancel
	return
    }
    global bltHierBox
    $widget selection $bltHierBox(action) anchor $node
    $widget selection anchor $node
}

# bind Hierbox <ButtonPress-1> { 
#     set bltHierBox(action) ""
#     blt::HierBoxSelect %W %x %y
# }
# bind Hierbox <Shift-ButtonPress-1> { 
#     set bltHierBox(action) "set"
#     %W selection anchor [%W nearest %x %y]
# }
# bind Hierbox <Control-ButtonPress-1> { 
#     set bltHierBox(action) "toggle"
#     %W selection anchor [%W nearest %x %y]
# }
# bind Hierbox <B1-Motion> { 
#     if { $bltHierBox(action) != "" } {
# 	set bltHierBox(x) %x
# 	set bltHierBox(y) %y
# 	blt::HierBoxMotion %W %x %y
#     } 
# }
# bind Hierbox <ButtonRelease-1> { 
#     if { $bltHierBox(action) != "" } {
# 	after cancel $bltHierBox(afterId)
# 	blt::HierBoxEndSelect %W %x %y
#     }
# }

bind Hierbox <ButtonPress-3> {
    %W hide [%W nearest %x %y]
}
bind Hierbox <Shift-ButtonPress-3> {
    %W show [%W nearest %x %y]
}

bind Hierbox <B2-Motion> {
    %W scan dragto @%x,%y
}

bind Hierbox <ButtonPress-2> {
    set bltHierBox(cursor) [%W cget -cursor]
    %W configure -cursor hand1
    %W scan mark @%x,%y
}

bind Hierbox <ButtonRelease-2> {
    %W configure -cursor $bltHierBox(cursor)
}

bind Hierbox <KeyPress-Up> {
    %W focus up
    %W see focus
    if { $bltHierBox(space) } {
	%W selection toggle focus
    }
}
bind Hierbox <KeyPress-Down> {
    %W focus down
    %W see focus
    if { $bltHierBox(space) } {
	%W selection toggle focus
    }
}

bind Hierbox <Shift-KeyPress-Up> {
    %W focus parent
    %W see focus
}
bind Hierbox <Shift-KeyPress-Down> {
    %W focus sibling
    %W see focus
}

bind Hierbox <B1-Leave> {
    blt::HierBoxAutoScan %W 
}

bind Hierbox <B1-Enter> {
    after cancel $bltHierBox(afterId)
}

# PageUp
bind Hierbox <KeyPress-Prior> {
    if { [%W index focus] != [%W index view.top] } {
	%W focus view.top
    } else {
	%W yview scroll -1 pages
	update
	%W focus view.bottom
    }
}
# PageDown
bind Hierbox <KeyPress-Next> {
    if { [%W index focus] != [%W index view.bottom] } {
	%W focus view.bottom
    } else {
	%W yview scroll 1 pages
	update
	%W focus view.top
    }
}

bind Hierbox <Control-KeyPress-l> {
    %W see focus
}
bind Hierbox <KeyPress-Left> {
    %W close focus
}
bind Hierbox <KeyPress-Right> {
    %W open focus
}

bind Hierbox <KeyPress-space> {
    %W selection toggle focus
    set bltHierBox(space) on
}

bind Hierbox <KeyRelease-space> { 
    set bltHierBox(space) off
}

bind Hierbox <KeyPress-Return> {
    %W selection toggle focus
    set bltHierBox(space) on
}

bind Hierbox <KeyRelease-Return> {
    set bltHierBox(space) off
}


bind Hierbox <Shift-KeyPress-Up> {
    %W focus up
    %W see focus
    %W selection set focus
}

bind Hierbox <Shift-KeyPress-Down> {
    %W focus down
    %W see focus
    %W selection set focus
}

bind Hierbox <Control-KeyPress-p> {
    %W focus up
    %W see focus
    %W selection toggle focus
}

bind Hierbox <Control-KeyPress-n> {
    %W focus down
    %W see focus
    %W selection toggle focus
}

bind Hierbox <KeyPress> {
    if { [string match {[A-Za-z0-9]*} "%A"] } {
	set last [%W index focus]
	set next [%W index next]
	while { $next != $last } {
	    set label [%W entry cget $next -label]
	    if { [string index $label 0] == "%A" } {
		break
	    }
	    set next [%W index -at $next next]
	}
	%W focus $next
	%W see focus
    }
}

# Home
bind Hierbox <KeyPress-Home> {
    %W focus root
    %W see focus
}
bind Hierbox <KeyPress-End> {
    %W focus end
    %W see focus
}

bind Hierbox <KeyPress-F1> {
    %W open -r root
}

bind Hierbox <KeyPress-F2> {
    eval %W close -r [%W entry children root 0 end] 
}

proc blt::InitEntryBindings { widget } {
    $widget bind all <Enter> { %W entry activate current }
    $widget bind all <Leave> { %W entry activate "" }
    $widget bind all <ButtonPress-1> { 
	set bltHierBox(action) ""

	# Clear all selections before setting the new anchor.
	%W selection clear 0 end
	%W selection set current
	%W selection anchor current

	%W see current
	%W focus current
    }

    $widget bind all <Shift-ButtonPress-1> { 
	set bltHierBox(action) "set"
	%W selection anchor current
    }

    $widget bind all <Control-ButtonPress-1> { 
	set bltHierBox(action) "toggle"
	%W selection anchor current
    }

    $widget bind all <B1-Motion> { 
	if { $bltHierBox(action) != "" } {
	    set bltHierBox(x) %x
	    set bltHierBox(y) %y
	    blt::HierBoxMotion %W %x %y
	} 
    }

    $widget bind all <ButtonRelease-1> { 
	if { $bltHierBox(action) != "" } {
	    after cancel $bltHierBox(afterId)
	    blt::HierBoxEndSelect %W %x %y
	}
    }
    $widget button bind all <ButtonRelease-1> {
	%W toggle current
    }
    $widget button bind all <Enter> {
	%W button activate current
    }
    $widget button bind all <Leave> {
	%W button activate ""
    }
}

